function D = dct2dict(sz_patch, n_atom)
%% dct2dict Overcomplete DCT dictionary for 2D image
%   D = dct2dict(sz_patch, n_atom) returns the overcomplete 2D DCT
%   dictionary with size (prod(sz_patch), n_atom).
%  
% The dictionary is generated by the tensor product of 1D filters, the columns of the top half of the 
% DCT matrix of double side in 1D case (equivalent to dctmtx() in MATLAB)
%  
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%Use of this code is free for research purposes only.
%
%Author:  Yuhui Quan
%
%Last Revision: 22-Jun-2014
 %
 %
    if numel(sz_patch)==1 sz_patch = [sz_patch sz_patch]; end
    
    n1 = sz_patch(1);
    N1 = ceil(sqrt(n_atom));
    D1 = zeros(n1,N1);
    D1(:,1) = 1/sqrt(n1);
    for k = 2: N1
        v = cos((0:n1-1)*pi*(k-1)/N1)';
        v = v-mean(v);
        D1(:,k) = v/norm(v);
    end
    n2 = sz_patch(2);
    N2 = n_atom/N1;
    D2 = zeros(n2,N2);
    D2(:,1) = 1/sqrt(n2);
    for k = 2: N2
        v = cos((0:n2-1)*pi*(k-1)/N2)';
        v = v-mean(v);
        D2(:,k) = v/norm(v);
    end
    D = kron(D1,D2);
    D = D(:,1:n_atom);
end
